define(['dojo/_base/declare', 'dijit/_WidgetBase', 'dijit/_TemplatedMixin', 'dijit/_WidgetsInTemplateMixin', 'jimu/utils', 'jimu/dijit/CheckBox', 'jimu/dijit/ColorPickerButton', 'jimu/dijit/Message', 'jimu/dijit/SimpleTable', 'dojo/_base/array', 'dojo/_base/html', 'dojo/_base/lang', 'dojo/dom-construct', 'dojo/on', 'dijit/form/TextBox', 'esri/Color', 'dojo/text!./SelectionSetting.html'], function (declare, _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, jimuUtils, Checkbox, ColorPickerButton, Message, SimpleTable, array, html, lang, domConstruct, on, TextBox, Color, SelectionSettingsTemplate) {
  return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {

    // Custom widget code goes here

    baseClass: 'spatial-prioritization-selection-settings',
    templateString: SelectionSettingsTemplate,

    nsl: null,
    layerDefinition: null,
    selectionDefinition: null,
    priorityField: null,
    selName: null,
    _selectionTable: null,
    _fieldList: null,
    _secondaryFieldList: null,
    _tertiaryFieldList: null,
    _rebuildTable: false,

    constructor: function constructor(options) {
      this.selectionDefinition = null;
      this.nls = null;
      this.layerDefinition = null;
      this.priorityField = null;
      this.selName = null;
      this.config = null;
      this.tr = null;
      lang.mixin(this, options);
    },
    postCreate: function postCreate() {
      this.inherited(arguments);

      this.selectionName.set('value', this.selName);
      this.own(on(this.primaryFieldSelect, 'change', lang.hitch(this, this._onChangePrimaryFieldSelect)));
      this.own(on(this.secondaryFieldSelect, 'change', lang.hitch(this, this._onChangeSecondaryFieldSelect)));

      this.enableSecondarySelection = new Checkbox({
        checked: false,
        label: this.nls.addSecondarySelection
      }, this.enableSecondarySelection);
      html.addClass(this.enableSecondarySelectionNode, 'esriCTHidden');
      this.enableTertiarySelection = new Checkbox({
        checked: false,
        label: this.nls.addTertiarySelection
      }, this.enableTertiarySelectionNode1);
      html.addClass(this.enableTertiarySelectionNode, 'esriCTHidden');
      this._onChangeSecondarySelection(false);
      this.own(on(this.enableSecondarySelection, 'change', lang.hitch(this, function () {
        this._onChangeSecondarySelection(this.enableSecondarySelection.getValue());
      })));
      this._onChangeTerciarySelection(false);
      this.own(on(this.enableTertiarySelection, 'change', lang.hitch(this, function () {
        this._onChangeTerciarySelection(this.enableTertiarySelection.getValue());
      })));

      this.secondaryOptionCheck = new Checkbox({
        checked: true,
        label: this.nls.secondaryOption
      }, this.secondaryOptionCheck);
      this.tertiaryOptionCheck = new Checkbox({
        checked: true,
        label: this.nls.tertiaryOption
      }, this.tertiaryOptionCheck);

      this._createFieldList(null);
      this._createSelectionTable();
    },
    destroy: function destroy() {
      this.inherited(arguments);
      this._layerDefinition = null;
      this.config = null;
      this.nls = null;
      this.tr = null;
    },
    setRelatedTr: function setRelatedTr(tr) {
      this.tr = tr;
    },
    getRelatedTr: function getRelatedTr() {
      return this.tr;
    },
    setDefinition: function setDefinition(definition) {
      this._selectionDefinition = definition || {};
    },
    getDefinition: function getDefinition() {
      return this._selectionDefinition;
    },
    setConfig: function setConfig(config) {
      if (Object.prototype.toString.call(config) !== "[object Object]") {
        return;
      }

      var selectionName = config.selectionName;
      if (!selectionName) {
        return;
      }

      this.config = config;
      this.shelter.show();
      this._rebuildTable = true;
      this.selectionName.set('value', config.selectionName);
      this.primaryFieldSelect.set('value', config.primaryField);
      this.enableSecondarySelection.setValue(config.useSecondaryField);
      this.secondaryOptionCheck.setValue(config.secondaryOption);
      var rows = config.selectionTable;

      setTimeout(lang.hitch(this, function () {
        this.secondaryFieldSelect.set('value', config.secondaryField);
        setTimeout(lang.hitch(this, function () {
          console.log('setting tertiary');
          this.enableTertiarySelection.setValue(config.useTertiaryField);
          this.tertiaryOptionCheck.setValue(config.tertiaryOption);
          this.tertiaryFieldSelect.set('value', config.tertiaryField);
        }), 200);
      }), 100);

      array.forEach(rows, lang.hitch(this, function (row) {
        var result = this._selectionTable.addRow({
          id: row.id
        });
        if (result.success && result.tr) {
          var tr = result.tr;
          this._addTextBox(tr, 1, row.label);
          this._addColorPickerButton(tr, 2, row.color);
          if (row.default) {
            tr.cells[3].childNodes[0].checked = true;
          }
        }
      }));
      this.shelter.hide();
    },
    getConfig: function getConfig() {
      var json = {
        selectionName: this.selectionName.get('value'),
        primaryField: this.primaryFieldSelect.get('value'),
        useSecondaryField: this.enableSecondarySelection.getValue(),
        secondaryField: this.secondaryFieldSelect.get('value'),
        secondaryOption: this.secondaryOptionCheck.getValue(),
        useTertiaryField: this.enableTertiarySelection.getValue(),
        tertiaryField: this.tertiaryFieldSelect.get('value'),
        tertiaryOption: this.tertiaryOptionCheck.getValue(),
        selectionTable: this._getSelectionTable()
      };

      return json;
    },
    _addTextBox: function _addTextBox(tr, pos, text) {
      var textBoxContainer = domConstruct.create("div", {
        "class": "esriCTLayerLabelTextBox"
      }, tr.cells[pos]);

      tr.textLabel = new TextBox({
        class: 'esriCTLayerLabelTextBox'
      }, textBoxContainer);
      tr.textLabel.set('value', text);
    },
    _addColorPickerButton: function _addColorPickerButton(tr, pos, color) {
      //from Near Me widget
      if (color === null) {
        color = new Color([Math.floor(Math.random() * 255) + 1, Math.floor(Math.random() * 255) + 1, Math.floor(Math.random() * 255) + 1, 0.75]);
      }
      var textBoxContainer = domConstruct.create("div", {
        "class": "esriCTLayerLabelTextBox"
      }, tr.cells[pos]);

      tr.colorPicker = new ColorPickerButton({
        color: new Color(color)
      }, textBoxContainer);
    },
    _createFieldList: function _createFieldList(selectedField) {
      var categoryFieldTypes = ['esriFieldTypeString', 'esriFieldTypeSmallInteger', 'esriFieldTypeInteger', 'esriFieldTypeSingle', 'esriFieldTypeDouble', 'esriFieldTypeDate'];
      var fieldInfos = lang.clone(this.layerDefinition.fields);

      var availableCategoryFieldInfos = array.filter(fieldInfos, lang.hitch(this, function (fieldInfo) {
        return categoryFieldTypes.indexOf(fieldInfo.type) >= 0 && !fieldInfo.name.includes('Shape_');
      }));

      this._fieldList = [{
        value: '',
        label: ' '
      }];
      array.forEach(availableCategoryFieldInfos, lang.hitch(this, function (fieldInfo) {
        var option = {
          value: fieldInfo.name,
          label: fieldInfo.alias
        };
        this._fieldList.push(option);
      }));
      this._fieldList = this._removeItemFromArray(this._fieldList, this.priorityField);
      this.primaryFieldSelect.set('options', this._fieldList);
      if (selectedField !== null) {
        this.primaryFieldSelect.set('value', selectedField);
      }

      this.primaryFieldSelect.startup();
    },
    _createSelectionTable: function _createSelectionTable() {
      var fields = [{
        name: 'id',
        title: this.nls.tableId,
        type: 'text',
        width: '150px'
      }, {
        name: 'label',
        title: this.nls.tableLabel,
        type: 'empty'
      }, {
        name: 'color',
        title: this.nls.tableColor,
        type: 'empty',
        editable: 'true',
        width: '60px'
      }, {
        name: 'default',
        title: this.nls.tableDefault,
        type: 'radio',
        editable: 'true',
        width: '70px',
        hidden: true
      }, {
        name: 'actions',
        title: '',
        width: '70px',
        type: 'actions',
        actions: ['up', 'down', 'delete']
      }];
      this._selectionTable = new SimpleTable({
        fields: fields,
        selectable: false
      }, this.selectionTableNode);
      this._selectionTable.startup();
      this.own(on(this._selectionTable, 'row-select', lang.hitch(this, this._onSelectionItemSelected)));
      this.own(on(this._selectionTable, 'row-delete', lang.hitch(this, this._onSelectionItemRemoved)));
      this.own(on(this._selectionTable, 'row-edit', lang.hitch(this, this._onSelectionItemEdit)));
    },
    _getSelectionTable: function _getSelectionTable() {
      var trs = this._selectionTable.getRows();
      var td = this._selectionTable.getData();
      var arraySelections = [];
      for (i = 0; i < trs.length; i++) {
        var itemSelection = {};
        itemSelection.id = td[i].id;
        itemSelection.default = td[i].default;
        itemSelection.color = trs[i].colorPicker.color;
        itemSelection.label = trs[i].textLabel.value;
        arraySelections.push(itemSelection);
      }

      return arraySelections;
    },
    _onBlurTextBox: function _onBlurTextBox(node) {
      node.set('value', jimuUtils.stripHTML(node.get('value')));
    },
    _onChangePrimaryFieldSelect: function _onChangePrimaryFieldSelect() {
      if (!this.primaryFieldSelect.options[0].value) {
        this.primaryFieldSelect.removeOption(0);
      }
      if (!this._rebuildTable) {
        this._selectionTable.clear();
        array.forEach(this.layerDefinition.fields, lang.hitch(this, function (field) {
          if (this.primaryFieldSelect.value === field.name) {
            if (field.domain !== null) {
              array.forEach(field.domain.codedValues, lang.hitch(this, function (codedValue) {
                result = this._selectionTable.addRow({
                  id: codedValue.code,
                  label: codedValue.name
                });
                if (result.success && result.tr) {
                  var tr = result.tr;
                  this._addTextBox(tr, 1, codedValue.name);
                  this._addColorPickerButton(tr, 2, null);
                }
              }));
            }
          }
        }));
      }
      this._secondaryFieldList = this._removeItemFromArray(this._fieldList, this.primaryFieldSelect.value);
      this.secondaryFieldSelect.removeOption(this.secondaryFieldSelect.getOptions());
      this.secondaryFieldSelect.set('options', this._secondaryFieldList);
      this.secondaryFieldSelect.startup();
      html.removeClass(this.enableSecondarySelectionNode, 'esriCTHidden');
      this._rebuildTable = false;
    },
    _onChangeSecondaryFieldSelect: function _onChangeSecondaryFieldSelect() {
      if (this.enableSecondarySelection.checked) {
        this._tertiaryFieldList = this._removeItemFromArray(this._secondaryFieldList, this.secondaryFieldSelect.value);
        this.tertiaryFieldSelect.removeOption(this.tertiaryFieldSelect.getOptions());
        this.tertiaryFieldSelect.set('options', this._tertiaryFieldList);
        this.tertiaryFieldSelect.startup();
        html.removeClass(this.enableTertiarySelectionNode, 'esriCTHidden');
      }
    },
    _onChangeSecondarySelection: function _onChangeSecondarySelection(enable) {
      if (enable) {
        html.removeClass(this.secondaryFieldNode, 'esriCTHidden');
      } else {
        html.addClass(this.secondaryFieldNode, 'esriCTHidden');
        this.secondaryFieldSelect.set('value', '');
        html.addClass(this.enableTertiarySelectionNode, 'esriCTHidden');
        this.enableTertiarySelection.setValue('checked', false);
      }
      this.secondaryFieldSelect.set('required', enable);
    },
    _onChangeTerciarySelection: function _onChangeTerciarySelection(enable) {
      if (enable) {
        html.removeClass(this.tertiaryFieldNode, 'esriCTHidden');
      } else {
        html.addClass(this.tertiaryFieldNode, 'esriCTHidden');
        this.tertiaryFieldSelect.set('value', '');
      }
      this.tertiaryFieldSelect.set('required', enable);
    },
    _onSelectionItemEdit: function _onSelectionItemEdit(tr) {
      console.log('Selection edited');
    },
    _onSelectionItemRemoved: function _onSelectionItemRemoved(tr) {},
    _onSelectionItemSelected: function _onSelectionItemSelected(tr) {
      console.log(tr);
    },
    _removeItemFromArray: function _removeItemFromArray(inputArray, inputItem) {
      return inputArray.filter(function (item) {
        return item.value !== inputItem;
      });
    },
    _setSelections: function _setSelections() {
      this._createFieldList(this.config.primaryField);
      if (this.config.useSecondaryField) {
        this.enableSecondarySelection.checked;
        this._onChangeSecondarySelection(true);
        this.secondaryFieldSelect.set('value', this.config.secondaryField);
      }
      if (this.config.useTertiaryField) {
        this.enableTertiarySelection.checked;
        this._onChangeTerciarySelection(true);
        this.tertiaryFieldSelect.set('value', this.config.tertiaryField);
      }
      array.forEach(this.config.selectionTable, lang.hitch(this, function (tr) {
        var result = this.selectionTable.addRow({});
      }));
    },
    _showMessage: function _showMessage(message) {
      new Message({
        message: message
      });
    }
  }

  // onOpen() {
  //   console.log('SpatialPrioritization::onOpen');
  // },
  // onClose(){
  //   console.log('SpatialPrioritization::onClose');
  // },
  // onMinimize(){
  //   console.log('SpatialPrioritization::onMinimize');
  // },
  // onMaximize(){
  //   console.log('SpatialPrioritization::onMaximize');
  // },
  // onSignIn(credential){
  //   console.log('SpatialPrioritization::onSignIn', credential);
  // },
  // onSignOut(
  //   console.log('SpatialPrioritization::onSignOut');
  // }
  // onPositionChange(){
  //   console.log('SpatialPrioritization::onPositionChange');
  // },
  // resize(){
  //   console.log('SpatialPrioritization::resize');
  // }

  );
});
//# sourceMappingURL=SelectionSetting.js.map
