define(['dojo/_base/declare', 'jimu/BaseWidgetSetting', "dijit/_WidgetsInTemplateMixin", "jimu/utils", "jimu/portalUtils", "jimu/dijit/_FeaturelayerSourcePopup", "jimu/dijit/ColorPickerButton", "jimu/dijit/Filter", "jimu/dijit/Message", "jimu/dijit/SimpleTable", "jimu/dijit/TabContainer3", "dojo/_base/array", "dojo/_base/html", "dojo/_base/lang", "dojo/dom-construct", "dojo/on", "dojo/query", "dijit/form/TextBox", "dijit/form/NumberTextBox", "dijit/form/ValidationTextBox", "esri/Color", "./SelectionSetting"], function (declare, BaseWidgetSetting, _WidgetsInTemplateMixin, jimuUtils, portalUtils, _FeaturelayerSourcePopup, ColorPickerButton, Filter, Message, SimpleTable, TabContainer3, array, html, lang, domConstruct, on, dojoQuery, TextBox, NumberTextBox, ValidationTextBox, Color, SelectionSetting) {
  return declare([BaseWidgetSetting, _WidgetsInTemplateMixin], {
    baseClass: 'spatial-prioritization-setting',
    _addOAuthTable: null, //store OAuth locations
    _portal: null,
    _layerSourceItem: null,
    _layerDefinition: null,
    _priorityField: null,
    _prioritizationMethod: null,
    _prioritizationTable: null,
    _settingConfig: false,

    postCreate: function postCreate() {
      // the config object is passed in
      this._initAgolSettings();
      this._initPrioritizationSettings();
      this._initFilterSettings();
      this._initAdditionalSettings();
      this.setConfig();
    },
    getConfig: function getConfig() {
      // AGOL Settings tab
      if (this._currentSelectionSetting) {
        this._closeSelectionSetting();
      }
      var config = void 0;
      config = {};
      config.agolSettings = {};
      config.agolSettings.ownerName = this.ownerName.value;
      config.agolSettings.baseGrid = this.baseGridLayerName.value;
      config.agolSettings.portalUrl = this.portalUrl.value;
      var auths = [];
      var authListTrs = this._addOAuthTable.getRows();
      array.forEach(authListTrs, lang.hitch(this, function (tr) {
        var auth = {
          location: tr.location.value,
          appId: tr.appId.value,
          popupCallbackUrl: tr.popupCallbackUrl.value
        };
        auths.push(auth);
      }));
      config.agolSettings.auths = auths;

      //Prioritization Settings tab
      if (this._layerDefinition === null) {
        this._errorMessage("You must select a Source Layer");
        return false;
      }
      config.prioritizationSettings = {};
      config.prioritizationSettings.method = this._prioritizationMethod;
      config.prioritizationSettings.sourceLayer = this.sourceLayerTextBox.value;
      config.prioritizationSettings.layerSourceItem = this._layerSourceItem;
      config.prioritizationSettings.field = this.sourceLayerFieldSelect.value;
      if (this._prioritizationMethod === "Coins") {
        config.prioritizationSettings.allocatedCoins = this.allocatedCoins.value;
        config.prioritizationSettings.maximumCoins = this.maximumCoins.value;
      }
      var prioritizationTd = this._prioritizationTable.getData();
      var prioritizationTrs = this._prioritizationTable.getRows();
      var prioritizations = [];

      //determine if id is text or number
      var fieldArray = array.filter(this._layerDefinition.fields, lang.hitch(this, function (field) {
        return field.name === this._priorityField;
      }));
      var categoryFieldTypes = ["esriFieldTypeSmallInteger", "esriFieldTypeInteger", "esriFieldTypeSingle", "esriFieldTypeDouble"];
      var isNumber = false;
      if (categoryFieldTypes.indexOf(fieldArray[0].type) > -1) {
        isNumber = true;
      }

      for (i = 0; i < prioritizationTrs.length; i++) {
        var id = prioritizationTd[i].id;
        if (isNumber) {
          id = Number(prioritizationTd[i].id);
        }
        var itemPriority = {
          id: id,
          label: prioritizationTrs[i].textLabel.value,
          max: prioritizationTrs[i].numberLabel.value,
          color: prioritizationTrs[i].colorPicker.color
        };
        prioritizations.push(itemPriority);
      }
      config.prioritizationSettings.prioritizations = prioritizations;

      //filter
      config.filter = this.filter.toJson();
      if (!config.filter) {}

      //Additional Settings tab
      var selections = [];
      var selectionListTrs = this.selectionList.getRows();
      array.forEach(selectionListTrs, lang.hitch(this, function (tr) {
        var selection = this._getRelatedConfig(tr);
        this._removeRelatedConfig(tr);
        selections.push(selection);
      }));
      config.selections = selections;
      return config;
    },
    setConfig: function setConfig(config) {
      this._settingConfig = true;
      if (this.config) {
        if (this.config.agolSettings) {
          this.ownerName.set("value", this.config.agolSettings.ownerName);
          this.baseGridLayerName.set("value", this.config.agolSettings.baseGrid);
          this.portalUrl.set("value", this.config.agolSettings.portalUrl);
          this._addOAuthTable.clear();
          array.forEach(this.config.agolSettings.auths, lang.hitch(this, function (auth) {
            this._addOAuthTableRow(auth.location, auth.appId, auth.popupCallbackUrl);
          }));
        }

        //Prioritization Settings tab
        if (this.config.prioritizationSettings) {
          if (this.config.prioritizationSettings.method === "Coins") {
            this.radPrioritizationCoins.checked = true;
            this.allocatedCoins.set("value", this.config.prioritizationSettings.allocatedCoins);
            this.maximumCoins.set("value", this.config.prioritizationSettings.maximumCoins);
          } else {
            this.radPrioritizationHigh.checked = true;
          }
          this._onPrioritizationMethodChange();
          if (this.config.prioritizationSettings.sourceLayer) {
            this.sourceLayerTextBox.set("value", this.config.prioritizationSettings.sourceLayer);
            this._layerSourceItem = this.config.prioritizationSettings.layerSourceItem;
            this._setPrioritizationTable(this._layerSourceItem);

            if (this.sourceLayerFieldSelect.getOptions(this.config.prioritizationSettings.field) !== null) {
              this.sourceLayerFieldSelect._setDisplay(this.config.prioritizationSettings.field);
              this._priorityField = this.config.prioritizationSettings.field;
              array.forEach(this.config.prioritizationSettings.prioritizations, lang.hitch(this, function (prioritization) {
                this._addPrioritizationTableRow(prioritization.id, prioritization.label, prioritization.max, prioritization.color);
              }));
              this._validateFilterTab();
              this._validateOtherSettingsTab();
            }
          }
        }

        if (this.config.filter) {
          this.filter.featureLayerId = this._layerDefinition.id;
          this.filter.buildByFilterObj(this._layerDefinition.url, this.config.filter, this._layerDefinition);
        }

        //Additional Settings tab
        if (this.config.selections) {
          array.forEach(this.config.selections, lang.hitch(this, function (selection, index) {
            var addResult = this.selectionList.addRow({
              selectionName: selection.selectionName || ""
            });
            if (addResult && addResult.success) {
              this._setRelatedConfig(addResult.tr, selection);
              if (index === 0) {
                var firstTr = addResult.tr;
                console.log("addResult", this.selectionList);
                setTimeout(lang.hitch(this, function () {
                  console.log("timeout", this.selectionList);
                  this.selectionList.selectRow(addResult.tr);
                  firstTr = null;
                }), 100);
              }
            } else {
              console.error("Add row failed in setConfig", addResult);
            }
          }));
        }
      }
      this._settingConfig = false;
    },
    _addAliasForLayerInfo: function _addAliasForLayerInfo(layerInfo) {
      if (layerInfo && layerInfo.fields && layerInfo.fields.length > 0) {
        array.forEach(layerInfo.fields, lang.hitch(this, function (fieldInfo) {
          if (fieldInfo.name && !fieldInfo.alias) {
            fieldInfo.alias = fieldInfo.name;
          }
        }));
      }
    },
    _addColorPickerButton: function _addColorPickerButton(tr, pos, color) {
      if (color === null) {
        color = new Color([this._getRandomInteger(0, 255), this._getRandomInteger(0, 255), this._getRandomInteger(0, 255),
        // Math.floor(Math.random() * 255) + 1,
        // Math.floor(Math.random() * 255) + 1,
        // Math.floor(Math.random() * 255) + 1,
        0.75]);
      }
      var textBoxContainer = domConstruct.create("div", {
        class: "esriCTLayerLabelTextBox"
      }, tr.cells[pos]);
      tr.colorPicker = new ColorPickerButton({
        color: new Color(color)
      }, textBoxContainer);
    },
    _addDomainNumberBox: function _addDomainNumberBox(tr, pos, number) {
      var numberContainer = domConstruct.create("div", {
        class: "esriCTLayerLabelTextBox"
      }, tr.cells[pos]);
      tr.numberLabel = new NumberTextBox({
        class: "esriCTLayerLabelTextBox"
      }, numberContainer);
      tr.numberLabel.set("value", number);
    },
    _addNumberBox: function _addNumberBox(tr, pos, number) {
      var numberContainer = domConstruct.create("div", {
        class: "esriCTLayerLabelTextBox"
      }, tr.cells[pos]);
      tr.numberLabel = new NumberTextBox({
        class: "esriCTLayerLabelTextBox",
        constraints: {
          min: 0,
          places: 0
        }
      }, numberContainer);
      tr.numberLabel.set("value", number);
    },
    _addOAuthTableRow: function _addOAuthTableRow(location, appId, popupCallbackUrl) {
      var result = this._addOAuthTable.addRow({}, this._addOAuthTable.getRows().length);
      if (result.success && result.tr) {
        var tr = result.tr;
        this._addTextBox(tr, 0, location, "location", "Data Source URL");
        this._addValidationTextBox(tr, 1, appId, "appId", "App ID", "^[a-zA-Z0-9]{16}$");
        this._addTextBox(tr, 2, popupCallbackUrl, "popupCallbackUrl", "Callback URL");
        //this._addValidationTextBox(tr, 2, popupCallbackUrl, 'popupCallbackUrl', 'Callback URL', '^(?:(?:https?|ftp):\/\/)(?:\S+(?::\S*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff]{2,}))\.?)(?::\d{2,5})?(?:[/?#]\S*)?$/i');
      }
    },
    _addPrioritizationTableRow: function _addPrioritizationTableRow(id, name, max, color) {
      var result = this._prioritizationTable.addRow({
        id: id
      });
      if (result.success && result.tr) {
        var tr = result.tr;
        this._addTextBox(tr, 1, name, "textLabel", "Dropdown label");
        this._addNumberBox(tr, 2, max);
        this._addColorPickerButton(tr, 3, color);
      }
    },
    _addTextBox: function _addTextBox(tr, pos, text, obj, placeHolder) {
      var textBoxContainer = domConstruct.create("div", {
        class: "esriCTLayerLabelTextBox"
      }, tr.cells[pos]);

      tr[obj] = new TextBox({
        class: "esriCTLayerLabelTextBox"
      }, textBoxContainer);
      tr[obj].set("value", text);
      tr[obj].set("placeHolder", placeHolder);
    },
    _addValidationTextBox: function _addValidationTextBox(tr, pos, text, obj, placeHolder, validation) {
      var textBoxContainer = domConstruct.create("div", {
        class: "esriCTLayerLabelTextBox"
      }, tr.cells[pos]);

      tr[obj] = new ValidationTextBox({
        class: "esriCTLayerLabelTextBox"
      }, textBoxContainer);
      tr[obj].set("value", text);
      tr[obj].set("placeHolder", placeHolder);
      tr[obj].set("regExp", validation);
    },
    _clearPrioritizationTable: function _clearPrioritizationTable() {
      this._layerDefinition = null;
      this.sourceLayerTextBox.set("value", null);
      this.sourceLayerFieldSelect.removeOption(this.sourceLayerFieldSelect.getOptions());
      this.sourceLayerFieldSelect._setDisplay("");
      this._validateOtherSettingsTab();
      this._priorityField = null;
    },
    _closeSelectionSetting: function _closeSelectionSetting() {
      var tr = this._currentSelectionSetting.getRelatedTr();
      var selection = this._currentSelectionSetting.getConfig();
      selection._definition = this._currentSelectionSetting.getDefinition();
      //check the rest of this code out
      this._setRelatedConfig(tr, selection);
      this.selectionList.editRow(tr, {
        selectionName: selection.selectionName
      });
      this._currentSelectionSetting.destroy();
    },
    _createAddOAuthGrid: function _createAddOAuthGrid() {
      var fields = [{
        name: "location",
        title: this.nls.agolSettings.location,
        type: "empty",
        width: "40%"
      }, {
        name: "appID",
        title: this.nls.agolSettings.appID,
        type: "empty",
        width: "20%"
      }, {
        name: "popupCallbackUrl",
        title: this.nls.agolSettings.popupCallbackUrl,
        type: "empty",
        width: "40%"
      }, {
        name: "actions",
        title: "",
        width: "30px",
        type: "actions",
        actions: ["delete"]
      }];
      var args = {
        fields: fields,
        selectable: false
      };
      this._addOAuthTable = new SimpleTable(args);
      this._addOAuthTable.placeAt(this.addOAuthTableNode);
      this._addOAuthTable.startup();
      this._onAddOAuthButtonClicked(true);
    },
    _createNewSelectionTableFromScratch: function _createNewSelectionTableFromScratch(tr, selectionName) {
      this._currentSelectionSetting = new SelectionSetting({
        nls: this.nls.otherSettings,
        layerDefinition: this._layerDefinition,
        priorityField: this._priorityField,
        selName: selectionName
      });

      this._currentSelectionSetting.placeAt(this.selectionSettingNode);
      this._currentSelectionSetting.setRelatedTr(tr);
    },
    _createNewSelectionTablefromList: function _createNewSelectionTablefromList(setting, definition, relatedTr) {
      if (this._currentSelectionSetting) {
        this._closeSelectionSetting();
      }

      this._currentSelectionSetting = new SelectionSetting({
        nls: this.nls.otherSettings,
        layerDefinition: this._layerDefinition,
        priorityField: this._priorityField
      });
      this._currentSelectionSetting.setDefinition(definition);
      this._currentSelectionSetting.setConfig({
        selectionName: setting.selectionName,
        primaryField: setting.primaryField,
        useSecondaryField: setting.useSecondaryField,
        secondaryField: setting.secondaryField,
        secondaryOption: setting.secondaryOption,
        useTertiaryField: setting.useTertiaryField,
        tertiaryField: setting.tertiaryField,
        tertiaryOption: setting.tertiaryOption,
        selectionTable: setting.selectionTable
      });

      this._currentSelectionSetting.placeAt(this.selectionSettingNode);

      this._currentSelectionSetting.setRelatedTr(relatedTr);
    },
    _createPrioritizationTable: function _createPrioritizationTable() {
      var hidden = void 0;
      //if using coins method
      if (this._prioritizationMethod === "Coins") {
        hidden = true;
      } else {
        hidden = false;
      }

      fields = [{
        name: "id",
        title: this.nls.prioritization.tableId,
        type: "text",
        editable: "false",
        width: "30%"
      }, {
        name: "label",
        title: this.nls.prioritization.tableLabel,
        type: "empty",
        editable: "true",
        width: "40%"
      }, {
        name: "max",
        title: this.nls.prioritization.tableMax,
        type: "empty",
        editable: "true",
        width: "10%",
        hidden: hidden
      }, {
        name: "color",
        title: this.nls.prioritization.tableColor,
        type: "empty",
        editable: "true",
        width: "10%"
      }, {
        name: "actions",
        title: "",
        width: "70px",
        type: "actions",
        actions: ["up", "down", "delete"]
      }];

      this._prioritizationTable = new SimpleTable({
        fields: fields,
        selectable: false
      });
      this._prioritizationTable.placeAt(this.prioritizationTableNode);
      this._prioritizationTable.startup();
    },
    _createSelectionListTable: function _createSelectionListTable() {
      this.selectionList = new SimpleTable({
        autoHeight: false,
        selectable: true,
        fields: [{
          name: "selectionName",
          title: "Selection",
          width: "auto",
          type: "text",
          editable: false
        }, {
          name: "actions",
          title: "",
          width: "70px",
          type: "actions",
          actions: ["up", "down", "delete"]
        }]
      }, this.selectionList);
      html.setStyle(this.selectionList.domNode, "height", "100%");
      this.selectionList.startup();
      this.own(on(this.selectionList, "row-select", lang.hitch(this, this._onSelectionItemSelected)));
      this.own(on(this.selectionList, "row-delete", lang.hitch(this, this._onSelectionItemRemoved)));
      this.selectionList.created = true;
    },
    _errorMessage: function _errorMessage(err) {
      new Message({
        message: err
      });
    },
    _getRandomInteger: function _getRandomInteger(min, max) {
      min = Math.ceil(min);
      max = Math.floor(max);
      return Math.floor(Math.random() * (max - min + 1)) + min;
    },
    _getRelatedConfig: function _getRelatedConfig(tr) {
      return dojoQuery(tr).data("config")[0];
    },
    _hasNumberFields: function _hasNumberFields() {
      //(layerDefinition) {
      var result = false;
      var fieldInfos = this._layerDefinition.fields;
      if (fieldInfos && fieldInfos.length > 0) {
        result = array.some(fieldInfos, lang.hitch(this, function (fieldInfo) {
          return this._numberFieldTypes.indexOf(fieldInfo.type) >= 0;
        }));
      }
      return result;
    },
    _initAdditionalSettings: function _initAdditionalSettings() {
      this._createSelectionListTable();
      this.own(on(this.btnNewSelection, "click", lang.hitch(this, this._onNewSelectionButtonClicked)));
    },
    _initAgolSettings: function _initAgolSettings() {
      this._initTabs();
      this._createAddOAuthGrid();

      this._portal = portalUtils.getPortal(this.appConfig.portalUrl);
      this.ownerName.set("value", this._portal.user.username);
      this.portalUrl.set("value", this._portal.portalUrl);

      this.own(on(this.btnAddOAuth, "click", lang.hitch(this, this._onAddOAuthButtonClicked)));
    },
    _initFilterSettings: function _initFilterSettings() {
      this.filter = new Filter({
        enableAskForValues: false,
        noFilterTip: this.nls.noFilterTip,
        style: "width:100%;"
      });
      this.filter.placeAt(this.filterReadyNode);
    },
    _initPrioritizationSettings: function _initPrioritizationSettings() {
      this.own(on(this.btnBaseLayerSet, "click", lang.hitch(this, this._setSourceLayerClicked)));
      this.own(on(this.sourceLayerFieldSelect, "change", lang.hitch(this, this._onPrioritizationFieldSelectChange)));
      this.own(on(this.allocatedCoins, "blur", lang.hitch(this, this._onAllocationCoinsBlur)));
      jimuUtils.combineRadioCheckBoxWithLabel(this.radPrioritizationCoins, this.labelPrioritizationCoins);
      jimuUtils.combineRadioCheckBoxWithLabel(this.radPrioritizationHigh, this.labelPrioritizationHigh);
      this.radPrioritizationCoins.checked = true;
      this._onPrioritizationMethodChange();
    },
    _initTabs: function _initTabs() {
      var agolSettingTab = {
        title: this.nls.agolSettings.agolSettingsTabLabel,
        content: this.agolSettingsNode
      };
      var prioritizationTab = {
        title: this.nls.prioritization.prioritizationSettingsTabLabel,
        content: this.prioritizationSettingsNode
      };
      var filterTab = {
        title: this.nls.filter.filterLabel,
        content: this.filterNode
      };
      var otherSettingsTab = {
        title: this.nls.otherSettings.otherSettingsTabLabel,
        content: this.otherSettingsNode
      };

      var tabs = [agolSettingTab, prioritizationTab, filterTab, otherSettingsTab];
      this._tab = new TabContainer3({
        tabs: tabs
      }, this.tabDiv);
      this._tab.startup();
    },
    _onAddOAuthButtonClicked: function _onAddOAuthButtonClicked(evt) {
      this._addOAuthTableRow(evt.type === "click" ? null : window.location.hostname, null, null);
    },
    _onAllocationCoinsBlur: function _onAllocationCoinsBlur() {
      this.maximumCoins.constraints.max = this.allocatedCoins.value;
    },
    _onHMLMethodChange: function _onHMLMethodChange() {
      if (this.radHMLPercent.checked) {
        //go through each row, get the Max Cells NumberTextBox, and change its constraints
      }
    },
    _onNewSelectionButtonClicked: function _onNewSelectionButtonClicked(clicked) {
      if (this._layerDefinition !== null) {
        var result = this.selectionList.addRow({
          selectionName: "Selection " + this.selectionList.getRows().length
        });
        if (result.success && result.tr) {
          var tr = result.tr;
          if (this._currentSelectionSetting) {
            this._closeSelectionSetting();
          }
          this._createNewSelectionTableFromScratch(tr, result.tr.cells[0].textContent);
        }
      } else {
        this._errorMessage(this.nls.otherSettings.baseLayerWarning);
      }
    },
    _onPrioritizationFieldSelectChange: function _onPrioritizationFieldSelectChange() {
      if (this._settingConfig) return;

      var drawingInfo = this._layerDefinition.drawingInfo;
      var fieldArray = array.filter(this._layerDefinition.fields, lang.hitch(this, function (field) {
        return field.name === this.sourceLayerFieldSelect.value;
      }));
      var field = fieldArray[0];
      this._prioritizationTable.clear();
      if (this.radPrioritizationCoins.checked) {
        if (drawingInfo.renderer.type === "uniqueValue" && drawingInfo.renderer.field1 === this.sourceLayerFieldSelect.value) {
          array.forEach(drawingInfo.renderer.uniqueValueInfos, lang.hitch(this, function (uniqueValueInfo) {
            this._addPrioritizationTableRow(uniqueValueInfo.value, uniqueValueInfo.label, null, uniqueValueInfo.symbol.color);
          }));
        } else {
          if (field.domain !== null) {
            array.forEach(field.domain.codedValues, lang.hitch(this, function (codedValue) {
              this._addPrioritizationTableRow(codedValue.code, codedValue.name, null, null);
            }));
          }
        }
      } else {
        if (drawingInfo.renderer.type === "uniqueValue" && drawingInfo.renderer.field1 === this.sourceLayerFieldSelect.value) {
          array.forEach(drawingInfo.renderer.uniqueValueInfos, lang.hitch(this, function (uniqueValueInfo) {
            this._addPrioritizationTableRow(uniqueValueInfo.value, uniqueValueInfo.label, 0, uniqueValueInfo.symbol.color);
          }));
        } else {
          if (field.domain !== null) {
            array.forEach(field.domain.codedValues, lang.hitch(this, function (codedValue) {
              this._addPrioritizationTableRow(codedValue.code, codedValue.name, 0, null);
            }));
          }
        }
      }
      this._priorityField = this.sourceLayerFieldSelect.value;
      this.selectionList.clear();

      this._validateOtherSettingsTab();
    },
    _onPrioritizationMethodChange: function _onPrioritizationMethodChange() {
      this._clearPrioritizationTable();
      if (this._prioritizationTable !== null) {
        this._prioritizationTable.destroy();
      }
      if (this.radPrioritizationCoins.checked) {
        html.removeClass(this.coinLimits, "esriCTHidden");
        html.addClass(this.HMLMethod, "esriCTHidden");
        this._prioritizationMethod = "Coins";
      } else {
        html.addClass(this.coinLimits, "esriCTHidden");
        this._prioritizationMethod = "Priority";
      }
      this._createPrioritizationTable();
      this._validateFilterTab();
    },
    _onSelectionItemRemoved: function _onSelectionItemRemoved(tr) {
      if (!this._currentSelectionSetting) {
        return;
      }

      var currentTr = this._currentSelectionSetting.getRelatedTr();
      if (currentTr === tr) {
        this._currentSelectionSetting.destroy();
        this._currentSelectionSetting = null;
      }
    },
    _onSelectionItemSelected: function _onSelectionItemSelected(tr) {
      var config = this._getRelatedConfig(tr);
      var currentTr = this._currentSelectionSetting && this._currentSelectionSetting.tr;
      if (!config || tr === currentTr) {
        return;
      }
      var rows = this.selectionList.getRows();
      var data = this.selectionList.getData();
      this._createNewSelectionTablefromList(config, config._definition || {}, tr);
    },
    _removeRelatedConfig: function _removeRelatedConfig(tr) {
      return dojoQuery(tr).removeData("config");
    },
    _resetPrioritizationFieldsByLayerInfo: function _resetPrioritizationFieldsByLayerInfo() {
      var fieldInfos = lang.clone(this._layerDefinition.fields);
      var categoryFieldTypes = void 0;
      if (this.radPrioritizationHigh.checked) {
        categoryFieldTypes = ["esriFieldTypeString", "esriFieldTypeSmallInteger", "esriFieldTypeInteger", "esriFieldTypeSingle", "esriFieldTypeDouble", "esriFieldTypeDate"];
      } else {
        categoryFieldTypes = ["esriFieldTypeSmallInteger", "esriFieldTypeInteger"];
      }
      var availableCategoryFieldInfos = array.filter(fieldInfos, lang.hitch(this, function (fieldInfo) {
        if (this.radPrioritizationHigh.checked) {
          return categoryFieldTypes.indexOf(fieldInfo.type) >= 0 && fieldInfo.domain !== null;
        } else {
          return categoryFieldTypes.indexOf(fieldInfo.type) >= 0 && !fieldInfo.name.includes("Shape_");
        }
      }));

      this.sourceLayerFieldSelect.removeOption(this.sourceLayerFieldSelect.getOptions());

      array.forEach(availableCategoryFieldInfos, lang.hitch(this, function (fieldInfo) {
        var option = {
          value: fieldInfo.name,
          label: fieldInfo.alias
        };

        this.sourceLayerFieldSelect.addOption(option);
      }));
      this.sourceLayerFieldSelect._setDisplay("");
      this._priorityField = null;
    },
    _resetPrioritizationTable: function _resetPrioritizationTable(sourceItem) {
      definition = sourceItem.definition;
      this._addAliasForLayerInfo(definition);
      this._clearPrioritizationTable();
      if (!definition) {
        return;
      }

      this._layerDefinition = definition;
      var url = definition.url;
      this.sourceLayerTextBox.set("value", url);

      this._resetPrioritizationFieldsByLayerInfo(); //(this._layerDefinition);
      this.sourceLayerFieldSelect.set("disabled", false);

      this.selectionSettingNode.removeAll;
    },
    _setPrioritizationTable: function _setPrioritizationTable(layerSourceItem) {
      layerSourceItem.definition.name = layerSourceItem.name;
      layerSourceItem.definition.url = layerSourceItem.url;
      var oldUrl = this._layerDefinition && this._layerDefinition.url;
      if (layerSourceItem.url !== oldUrl) {
        this._resetPrioritizationTable(layerSourceItem);
      }
    },
    _setRelatedConfig: function _setRelatedConfig(tr, selection) {
      dojoQuery(tr).data("config", lang.clone(selection));
    },
    _setSourceLayerClicked: function _setSourceLayerClicked() {
      var args = {
        titleLabel: this.nls.prioritization.setDataSourceTitle,
        dijitArgs: {
          multiple: false,
          createMapResponse: this.map.webMapResponse,
          portalUrl: this.appConfig.portalUrl,
          style: {
            height: "100%"
          }
        }
      };

      var featurePopup = new _FeaturelayerSourcePopup(args);
      this.own(on(featurePopup, "ok", lang.hitch(this, function (item) {
        featurePopup.close();

        var layerName = item.name || "";
        this._layerSourceItem = item;
        this._setPrioritizationTable(item);
        this.filter.featureLayerId = this._layerDefinition.id;
        this.filter.buildByExpr(this._layerDefinition.url, "1=1", this._layerDefinition);
        this._validateFilterTab();
      })));

      this.own(on(featurePopup, "cancel", lang.hitch(this, function () {
        featurePopup.close();
      })));
      featurePopup.startup();
    },
    _validateConfig: function _validateConfig(config) {
      if (!config.agol.ownerName) {
        this._errorMessage("Invalid Owner Name");
        return false;
      }
      return true;
    },
    _validateFilterTab: function _validateFilterTab() {
      if (this._layerDefinition) {
        html.removeClass(this.filterReadyNode, "esriCTHidden");
        html.addClass(this.filterNotReadyNode, "esriCTHidden");
      } else {
        html.removeClass(this.filterNotReadyNode, "esriCTHidden");
        html.addClass(this.filterReadyNode, "esriCTHidden");
      }
    },
    _validateOtherSettingsTab: function _validateOtherSettingsTab() {
      if (this._layerDefinition && this.sourceLayerFieldSelect.value !== "") {
        html.removeClass(this.settingsReadyNode, "esriCTHidden");
        html.addClass(this.settingsNotReadyNode, "esriCTHidden");
        html.removeClass(this.filterReadyNode, "esriCTHidden");
        html.addClass(this.filterNotReadyNode, "esriCTHidden");
      } else {
        //clean out Selection Table
        if (this.selectionList.created) {
          this.selectionList.clear();
        }
        html.removeClass(this.settingsNotReadyNode, "esriCTHidden");
        html.addClass(this.settingsReadyNode, "esriCTHidden");
        html.removeClass(this.filterNotReadyNode, "esriCTHidden");
        html.addClass(this.filterReadyNode, "esriCTHidden");
      }
    }
  });
});
//# sourceMappingURL=Setting.js.map
